#include <set>
#include "fx.h"

#define TITLE "The Analyzer"

class AnalyzerWindow : public FXMainWindow
{
FXDECLARE(AnalyzerWindow)
protected:
  FXString filename;
  FXMenuBar* menubar;
  FXMenuPane* filemenu;
  FXStatusBar* statusbar;
  FXText* editor;
  FXDirList* dirlist;
  FXIconList* iconlist;
  FXHiliteStyle stylecolors[1];
protected:
  AnalyzerWindow() { }
  bool analyze(const FXString& infile, const FXString& outfile);
  bool openfile(const FXString& infile);
  bool savefile(const FXString& outfile);
public:
  enum
  {
    ID_QUIT = FXMainWindow::ID_LAST,
    ID_OPEN,
    ID_SAVE,
    ID_DIRLIST
  };
public:
  long onUpdate(FXObject*,FXSelector,void*);
  long onCmdQuit(FXObject*,FXSelector,void*);
  long onCmdOpen(FXObject*,FXSelector,void*);
  long onCmdSave(FXObject*,FXSelector,void*);
  long onCmdDirList(FXObject*,FXSelector,void*);
public:
  AnalyzerWindow(FXApp* app);

  virtual void create();

  virtual ~AnalyzerWindow();
};

FXDEFMAP(AnalyzerWindow) AnalyzerWindowMap[] = {
  FXMAPFUNC(SEL_UPDATE, 0, AnalyzerWindow::onUpdate),
  FXMAPFUNC(SEL_CLOSE, 0, AnalyzerWindow::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND, AnalyzerWindow::ID_QUIT, AnalyzerWindow::onCmdQuit),
  FXMAPFUNC(SEL_COMMAND, AnalyzerWindow::ID_OPEN, AnalyzerWindow::onCmdOpen),
  FXMAPFUNC(SEL_COMMAND, AnalyzerWindow::ID_SAVE, AnalyzerWindow::onCmdSave),
  FXMAPFUNC(SEL_COMMAND, AnalyzerWindow::ID_DIRLIST, AnalyzerWindow::onCmdDirList)
};

FXIMPLEMENT(AnalyzerWindow, FXMainWindow, AnalyzerWindowMap, ARRAYNUMBER(AnalyzerWindowMap))

AnalyzerWindow::AnalyzerWindow(FXApp* app)
: FXMainWindow(app, TITLE, NULL, NULL, DECOR_ALL, 0, 0, 800, 600, 0, 0),
  filename(FXString::null)
{
  stylecolors[0].normalForeColor = FXRGB(255,255,255);
  stylecolors[0].normalBackColor = FXRGB(255, 0, 0);

  statusbar = new FXStatusBar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER|FRAME_RAISED);

  menubar = new FXMenuBar(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|FRAME_RAISED);

  filemenu=new FXMenuPane(this);

  new FXMenuTitle(menubar, "&File", NULL, filemenu);
  new FXMenuCommand(filemenu, "&Open...\tCtl-O\tOpen file.", NULL, this, ID_OPEN);
  new FXMenuCommand(filemenu, "&Save As...\tCtl-S\tSave file.", NULL, this, ID_SAVE);
  new FXMenuSeparator(filemenu);
  new FXMenuCommand(filemenu, "&Quit\tCtl-Q\tExit application.", NULL, this, ID_QUIT);

  FXSplitter* split1 = new FXSplitter(this, SPLITTER_TRACKING|LAYOUT_FILL_X|LAYOUT_FILL_Y);

  FXVerticalFrame* left = new FXVerticalFrame(split1, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);

  FXVerticalFrame* right = new FXVerticalFrame(split1, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);

  dirlist=new FXDirList(left, this, ID_DIRLIST, DIRLIST_SHOWFILES|DIRLIST_NO_OWN_ASSOC|TREELIST_BROWSESELECT|TREELIST_SHOWS_LINES|TREELIST_SHOWS_BOXES|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  dirlist->setPattern("*.h,*.c");
  dirlist->setDirectory(FXSystem::getCurrentDirectory());

  FXSplitter* split2 = new FXSplitter(right, SPLITTER_REVERSED|SPLITTER_VERTICAL|SPLITTER_TRACKING|LAYOUT_FILL_X|LAYOUT_FILL_Y);

  FXVerticalFrame* top = new FXVerticalFrame(split2, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);

  FXVerticalFrame* bottom = new FXVerticalFrame(split2, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0,0,0,0, 0,0,0,0);

  editor=new FXText(top, NULL, 0, TEXT_SHOWACTIVE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  editor->setHiliteMatchTime(2000);
  editor->setBarColumns(6);
  editor->setBarColor(FXRGB(240, 240, 240));
  editor->setActiveBackColor(FXRGB(240, 240, 240));
  editor->setHiliteStyles(stylecolors);
  editor->setStyled(TRUE);

  iconlist=new FXIconList(bottom, NULL, 0, ICONLIST_DETAILED|ICONLIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);
  iconlist->appendHeader("Location", NULL, 80);
  iconlist->appendHeader("Declared", NULL, 80);
  iconlist->appendHeader("Message", NULL, 1500);

  split1->setSplit(0, 200);
  split2->setSplit(1, 150);
}

void AnalyzerWindow::create()
{
  FXMainWindow::create();
  show(PLACEMENT_SCREEN);
}

bool AnalyzerWindow::analyze(const FXString& infile, const FXString& outfile)
{
  FXString middle = infile + ".i";
  FXString cmd = FXStringFormat("./lcc-cpp %s > %s", infile.text(), middle.text());
  int result = system(cmd.text());

  if(result != 0)
    return false;

  cmd = FXStringFormat("./lcc-rcc -target=null %s > %s", middle.text(), outfile.text());
  result = system(cmd.text());

  if(result != 0)
    return false;

  return true;
}

bool AnalyzerWindow::openfile(const FXString& infile)
{
  FXString outfile = infile + ".out";

  getApp()->beginWaitCursor();

  // Perform analysis
  if(!analyze(infile, outfile))
  {
    getApp()->endWaitCursor();
    FXMessageBox::error(this, MBOX_OK, "Error Analyzing File", "An error occured when analyzing file: %s", infile.text());
    return false;
  }

  // Load the analysis
  FXFile resultfile(outfile, FXFile::Reading);
  FXString str;
  FXchar c;
  std::set<FXint> errors;

  if(!resultfile.isOpen())
  {
    getApp()->endWaitCursor();
    FXMessageBox::error(this, MBOX_OK, "Error Loading File", "Unable to open file: %s", outfile.text());
    return false;
  }

  // Clear out the old data
  iconlist->clearItems();

  resultfile.readBlock(&c, 1);
  str.append(c);
  while(!resultfile.eof())
  {
    if(c == '\n')
    {
      str.trim();

      if(str.contains(':') == 2)
      {
        FXString txt = str.before(':'), tmp = str.after(':');

        errors.insert(FXIntVal(txt));

        txt.append('\t');
        txt += tmp.before(':');

        txt.append('\t');
        txt += tmp.after(':');

        iconlist->appendItem(txt);
      }
      else
      {
        str.prepend('\t', 2);
        iconlist->appendItem(str);
      }

      str.clear();
    }

    resultfile.readBlock(&c, 1);
    str.append(c);
  }

  // In case their was no new line before eof
  if(!str.empty())
  {
    str.trim();

    if(str.contains(':') == 2)
    {
      FXString txt = str.before(':'), tmp = str.after(':');

      errors.insert(FXIntVal(txt));

      txt.append('\t');
      txt += tmp.before(':');

      txt.append('\t');
      txt += tmp.after(':');

      iconlist->appendItem(txt);
    }
    else
    {
      str.prepend('\t', 2);
      iconlist->appendItem(str);
    }

    str.clear();
  }


  // Load the file
  FXFile textfile(infile, FXFile::Reading);
  FXint lineno = 0;

  if(!textfile.isOpen())
  {
    getApp()->endWaitCursor();
    FXMessageBox::error(this, MBOX_OK, "Error Loading File", "Unable to open file: %s", infile.text());
    return false;
  }

  // Clear the text area
  editor->setText(FXString::null);

  textfile.readBlock(&c, 1);
  str.append(c);
  while(!textfile.eof())
  {
    if(c == '\n')
    {
      lineno++;
      if(errors.find(lineno) != errors.end())
      {
        editor->appendStyledText(str, 1);
      }
      else
      {
        editor->appendText(str);
      }

      str.clear();
    }

    textfile.readBlock(&c, 1);
    str.append(c);
  }

  // In case their was no new line before eof
  if(!str.empty())
  {
    lineno++;
    if(errors.find(lineno) != errors.end())
    {
      editor->appendStyledText(str, 1);
    }
    else
    {
      editor->appendText(str);
    }

    str.clear();
  }

  getApp()->endWaitCursor();

  return true;
}

bool AnalyzerWindow::savefile(const FXString& outfile)
{
  FXint size,n;
  FXchar *text;
  FXFile textfile(outfile, FXFile::Writing);

  if(!textfile.isOpen())
  {
    FXMessageBox::error(this, MBOX_OK, "Error Saving File", "Unable to open file for writing: %s", outfile.text());
    return false;
  }

  size=editor->getLength();

  if(!FXMALLOC(&text, FXchar, size+1))
  {
    FXMessageBox::error(this, MBOX_OK, "Error Saving File", "File is too large: %s", outfile.text());
    return false;
  }

  getApp()->beginWaitCursor();

  editor->getText(text, size);

  n = textfile.writeBlock(text, size);

  FXFREE(&text);

  getApp()->endWaitCursor();

  if(n != size)
  {
    FXMessageBox::error(this, MBOX_OK, "Error Saving File", "Not enough space on disk: %s", outfile.text());
    return false;
  }

  return true;
}

long AnalyzerWindow::onUpdate(FXObject* sender, FXSelector sel, void* ptr)
{
  FXMainWindow::onUpdate(sender, sel, ptr);

  if(!filename.empty())
    setTitle(FXString(TITLE) + FXString(" - ") + FXString(filename));
  else
    setTitle(FXString(TITLE));

  return 1;
}

long AnalyzerWindow::onCmdQuit(FXObject*,FXSelector,void*)
{
   getApp()->exit(0);
  return 1;
}

long AnalyzerWindow::onCmdOpen(FXObject*,FXSelector,void*)
{
  FXFileDialog opendialog(this, "Open File");

  opendialog.setSelectMode(SELECTFILE_EXISTING);
  opendialog.setPatternList("C Files (*.c,*.h)");
  opendialog.setDirectory(dirlist->getDirectory());

  if(opendialog.execute(PLACEMENT_OWNER))
  {
    FXString file=opendialog.getFilename();

    if(openfile(file))
      filename = file;
  }

  return 1;
}

long AnalyzerWindow::onCmdSave(FXObject*,FXSelector,void*)
{
  FXFileDialog savedialog(this, "Save File");

  savedialog.setSelectMode(SELECTFILE_ANY);
  savedialog.setPatternList("C Files (*.c,*.h)");
  savedialog.setFilename(filename);

  if(savedialog.execute(PLACEMENT_OWNER))
  {
    FXString file=savedialog.getFilename();

    if(FXStat::exists(file))
    {
      if(MBOX_CLICKED_NO==FXMessageBox::question(this, MBOX_YES_NO, "Overwrite File", "Overwrite existing file: %s?", file.text()))
        return 1;
    }
    savefile(file);
  }

  return 1;
}

long AnalyzerWindow::onCmdDirList(FXObject*,FXSelector,void*)
{
  FXString file = dirlist->getCurrentFile();

  if(FXStat::isFile(file) && openfile(file))
    filename = file;

  return 1;
}

int main(int argc,char **argv)
{
  FXApp app("InitializationAnalyzer");

  app.init(argc,argv);

  new AnalyzerWindow(&app);

  app.create();

  return app.run();
}

AnalyzerWindow::~AnalyzerWindow()
{
  delete filemenu;
}
